@@ -2,7 +2,7 @@
 #     Project: PyOdoo
 # Description: API for Odoo
 #      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
-#   Copyright: 2021-2022 Fabio Castelli
+#   Copyright: 2021-2023 Fabio Castelli
 #     License: GPL-3+
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -137,11 +137,27 @@ class Api(object):
         :param options: Dictionary with options to use
         :return: Dictionary with the requested fields
         """
-        results = self.do_execute(method_name='read',
-                                  args=[entity_id],
-                                  kwargs=options)
+        results = self.do_read_many(entity_ids=[entity_id],
+                                    options=options)
         return results[0] if results else None
 
+    def do_read_many(self,
+                     entity_ids: list[int],
+                     options: dict[str, Any]
+                     ) -> Optional[list[dict[str, Any]]]:
+        """
+        Get multiple records in the requested model applying a filter and some
+        options (like fields or context)
+
+        :param entity_ids: Object IDs to get
+        :param options: Dictionary with options to use
+        :return: List of dictionaries with the requested fields
+        """
+        results = self.do_execute(method_name='read',
+                                  args=[entity_ids],
+                                  kwargs=options)
+        return results
+
     def do_fields_get(self,
                       fields: list[str],
                       options: dict[str, Any]) -> dict[str, dict]:
@@ -221,31 +237,31 @@ class Api(object):
         return results
 
     def do_update(self,
-                  entity_id: int,
+                  entity_id: Union[int, list[int]],
                   values: dict[str, Any],
                   options: dict[str, Any]) -> bool:
         """
-        Update a record in the requested model
+        Update one or more records in the requested model
 
-        :param entity_id: object ID to update
+        :param entity_id: The Object IDs to update
         :param values: Dictionary with the fields to update and their values
         :param options: Dictionary with options to use
-        :return: True if the record was updated
+        :return: True if the records were updated
         """
         results = self.do_execute(method_name='write',
-                                  args=[[entity_id], values],
+                                  args=[entity_id, values],
                                   kwargs=options)
         return results
 
     def do_delete(self,
-                  entity_id: int,
+                  entity_id: Union[int, list[int]],
                   options: dict[str, Any]) -> bool:
         """
-        Delete a record in the requested model
+        Delete one or multiple records in the requested model
 
-        :param entity_id: Object ID to get
+        :param entity_id: The Object IDs to delete
         :param options: Dictionary with options to use
-        :return: True if the record was updated
+        :return: True if the records were deleted
         """
         results = self.do_execute(method_name='unlink',
                                   args=[entity_id],
