@@ -2,7 +2,7 @@
 #     Project: PyOdoo
 # Description: API for Odoo
 #      Author: Fabio Castelli (Muflone) <muflone@muflone.com>
-#   Copyright: 2021-2022 Fabio Castelli
+#   Copyright: 2021-2023 Fabio Castelli
 #     License: GPL-3+
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 ##
 
 from typing import Any, Optional, Union
+import xmlrpc
 
 from pyodoo import (ActiveStatusChoice,
                     BooleanOperator,
@@ -38,7 +39,8 @@ class Model(object):
                  database: str,
                  username: str,
                  password: str,
-                 language: str = None):
+                 language: str = None,
+                 authenticate: bool = False):
         # API object
         self.api = Api(model_name=model_name,
                        endpoint=endpoint,
@@ -48,6 +50,9 @@ class Model(object):
                        language=language)
         # Message Subtype IDs
         self._message_subtypes = {}
+        # Automatically authenticate if required
+        if authenticate:
+            self.authenticate()
 
     @property
     def model_name(self):
@@ -86,10 +91,12 @@ class Model(object):
         return self.api.authenticate()
 
     def get_model(self,
-                  model_name: str) -> 'Model':
+                  model_name: str,
+                  authenticate: bool = False) -> 'Model':
         """
         Get a Model object for another model name
         :param model_name: Model name
+        :param authenticate: Automatically authenticate user
         :return: Model object
         """
         return Model(model_name=model_name,
@@ -97,7 +104,8 @@ class Model(object):
                      database=self.api.database,
                      username=self.api.username,
                      password=self.api.password,
-                     language=self.api.language)
+                     language=self.api.language,
+                     authenticate=authenticate)
 
     def get_model_data_reference(self,
                                  module_name: str,
@@ -109,7 +117,8 @@ class Model(object):
         :param value: Value to lookup
         :return: Dictionary with the data for the referenced object
         """
-        model = self.get_model(model_name='ir.model.data')
+        model = self.get_model(model_name='ir.model.data',
+                               authenticate=True)
         filters = [Filter(field='module',
                           compare_type=CompareType.EQUAL,
                           value=module_name),
@@ -117,10 +126,9 @@ class Model(object):
                           compare_type=CompareType.EQUAL,
                           value=value)]
         options = {}
-        model.set_pagination(options=options,
-                             limit=1,
-                             offset=0)
-        model.authenticate()
+        model._set_pagination(options=options,
+                              limit=1,
+                              offset=0)
         results = model.filter(filters=filters,
                                fields=(),
                                options={},
@@ -146,26 +154,55 @@ class Model(object):
         :param order: Ordering clause
         :return: Dictionary with the requested fields
         """
+        results = self.get_many(entity_ids=[entity_id],
+                                fields=fields,
+                                options=options,
+                                limit=limit,
+                                offset=offset,
+                                order=order)
+        return results[0] if results else None
+
+    def get_many(self,
+                 entity_ids: list[int],
+                 fields: tuple[str, ...] = None,
+                 options: dict[str, Any] = None,
+                 limit: Optional[int] = None,
+                 offset: Optional[int] = None,
+                 order: Optional[str] = None
+                 ) -> Optional[list[dict[str, Any]]]:
+        """
+        Get a row from a model using its ID
+
+        :param entity_ids: Object IDs to query
+        :param fields: Tuple with the fields to include in the response
+        :param options: Dictionary with options to use
+        :param limit: Maximum number of results count
+        :param offset: Starting record number to fetch the data
+        :param order: Ordering clause
+        :return: List of dictionaries with the requested fields
+        """
+        # Set options
         if options is None:
             options = {}
         # Limit results only to selected fields
         if fields:
             options['fields'] = fields
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Set pagination
-        self.set_pagination(options=options,
-                            limit=limit,
-                            offset=offset)
+        self._set_pagination(options=options,
+                             limit=limit,
+                             offset=offset)
         # Set order
-        self.set_order_by(options=options,
-                          order=order)
+        self._set_order_by(options=options,
+                           order=order)
         # Request data and get results
-        results = self.api.do_read(entity_id=entity_id,
-                                   options=options)
+        results = self.api.do_read_many(entity_ids=entity_ids,
+                                        options=options)
         return results
 
     def all(self,
+            is_active: ActiveStatusChoice = ActiveStatusChoice.NOT_SET,
             fields: tuple[str, ...] = None,
             options: dict[str, Any] = None,
             limit: Optional[int] = None,
@@ -174,6 +211,7 @@ class Model(object):
         """
         Get all the objects
 
+        :param is_active: Additional filter for active field
         :param fields: Fields to include in the response
         :param options: Dictionary with options to use
         :param limit: Maximum number of results count
@@ -182,6 +220,7 @@ class Model(object):
         :return: List of dictionaries with the requested fields
         """
         return self.filter(filters=[],
+                           is_active=is_active,
                            fields=fields,
                            options=options,
                            limit=limit,
@@ -190,8 +229,8 @@ class Model(object):
 
     def find(self,
              entity_ids: list[int],
-             fields: tuple[str, ...] = None,
              is_active: ActiveStatusChoice = ActiveStatusChoice.NOT_SET,
+             fields: tuple[str, ...] = None,
              options: dict[str, Any] = None,
              limit: Optional[int] = None,
              offset: Optional[int] = None,
@@ -200,8 +239,8 @@ class Model(object):
         Find some rows from a model using their ID
 
         :param entity_ids: Objects ID to query
-        :param fields: Tuple with the fields to include in the response
         :param is_active: Additional filter for active field
+        :param fields: Tuple with the fields to include in the response
         :param options: Dictionary with options to use
         :param limit: Maximum number of results count
         :param offset: Starting record number to fetch the data
@@ -210,25 +249,24 @@ class Model(object):
         """
         # Add filtered IDs
         filters = [['id', CompareType.IN, entity_ids]]
-        # Check for active records (Only active, only inactive or both)
-        if is_active == ActiveStatusChoice.BOTH:
-            filters.append(['active', CompareType.IN, is_active])
-        elif is_active != ActiveStatusChoice.NOT_SET:
-            filters.append(['active', CompareType.EQUAL, is_active])
+        # Filter for active status
+        self._set_active(filters=filters,
+                         is_active=is_active)
+        # Set options
         if options is None:
             options = {}
         # Limit results only to selected fields
         if fields:
             options['fields'] = fields
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Set pagination
-        self.set_pagination(options=options,
-                            limit=limit,
-                            offset=offset)
+        self._set_pagination(options=options,
+                             limit=limit,
+                             offset=offset)
         # Set order
-        self.set_order_by(options=options,
-                          order=order)
+        self._set_order_by(options=options,
+                           order=order)
         # Request data and get results
         results = self.api.do_search_read(filters=filters,
                                           options=options)
@@ -236,6 +274,7 @@ class Model(object):
 
     def filter(self,
                filters: list[Union[BooleanOperator, Filter]],
+               is_active: ActiveStatusChoice = ActiveStatusChoice.NOT_SET,
                fields: tuple[str, ...] = None,
                options: dict[str, Any] = None,
                limit: Optional[int] = None,
@@ -245,6 +284,7 @@ class Model(object):
         Find some rows from a model using some filters
 
         :param filters: List of filters used for searching the data
+        :param is_active: Additional filter for active field
         :param fields: Tuple with the fields to include in the response
         :param options: Dictionary with options to use
         :param limit: Maximum number of results count
@@ -252,20 +292,24 @@ class Model(object):
         :param order: Ordering clause
         :return: List of dictionaries with the requested fields
         """
+        # Filter for active status
+        self._set_active(filters=filters,
+                         is_active=is_active)
+        # Set options
         if options is None:
             options = {}
         # Limit results only to selected fields
         if fields:
             options['fields'] = fields
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Set pagination
-        self.set_pagination(options=options,
-                            limit=limit,
-                            offset=offset)
+        self._set_pagination(options=options,
+                             limit=limit,
+                             offset=offset)
         # Set order
-        self.set_order_by(options=options,
-                          order=order)
+        self._set_order_by(options=options,
+                           order=order)
         # Request data and get results
         results = self.api.do_search_read(filters=filters,
                                           options=options)
@@ -273,14 +317,20 @@ class Model(object):
 
     def count(self,
               filters: list[Union[BooleanOperator, Filter]],
+              is_active: ActiveStatusChoice = ActiveStatusChoice.NOT_SET,
               options: dict[str, Any] = None) -> int:
         """
         Get the rows count from a model using some filters
 
         :param filters: List of filters used for searching the data
+        :param is_active: Additional filter for active field
         :param options: Dictionary with options to use
         :return: Rows count
         """
+        # Filter for active status
+        self._set_active(filters=filters,
+                         is_active=is_active)
+        # Set options
         if options is None:
             options = {}
         # Request data and get results
@@ -290,6 +340,7 @@ class Model(object):
 
     def search(self,
                filters: list[Union[BooleanOperator, Filter]],
+               is_active: ActiveStatusChoice = ActiveStatusChoice.NOT_SET,
                options: dict[str, Any] = None,
                limit: Optional[int] = None,
                offset: Optional[int] = None,
@@ -298,23 +349,28 @@ class Model(object):
         Find rows list from a list of filters
 
         :param filters: List of filters used for searching the data
+        :param is_active: Additional filter for active field
         :param options: Dictionary with options to use
         :param limit: Maximum number of results count
         :param offset: Starting record number to fetch the data
         :param order: Ordering clause
         :return: List of ID for the objects found
         """
+        # Filter for active status
+        self._set_active(filters=filters,
+                         is_active=is_active)
+        # Set options
         if options is None:
             options = {}
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Set pagination
-        self.set_pagination(options=options,
-                            limit=limit,
-                            offset=offset)
+        self._set_pagination(options=options,
+                             limit=limit,
+                             offset=offset)
         # Set order
-        self.set_order_by(options=options,
-                          order=order)
+        self._set_order_by(options=options,
+                           order=order)
         # Request data and get results
         results = self.api.do_search(filters=filters,
                                      options=options)
@@ -330,31 +386,33 @@ class Model(object):
         :param options: Dictionary with options to use
         :return: The ID of the newly created object
         """
+        # Set options
         if options is None:
             options = {}
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Create data and get results
         results = self.api.do_create(values=values,
                                      options=options)
         return results
 
     def update(self,
-               entity_id: int,
+               entity_id: Union[int, list[int]],
                values: dict[str, Any],
                options: dict[str, Any] = None) -> bool:
         """
-        Get a row from a model using its ID
+        Update one or multiple rows from a model using the object IDs
 
-        :param entity_id: The object ID to update
+        :param entity_id: The object IDs to update
         :param values: Dictionary with the fields to update and their values
         :param options: Dictionary with options to use
-        :return: True if the record was updated
+        :return: True if the records were updated
         """
+        # Set options
         if options is None:
             options = {}
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Update data and get results
         results = self.api.do_update(entity_id=entity_id,
                                      values=values,
@@ -362,15 +420,16 @@ class Model(object):
         return results
 
     def delete(self,
-               entity_id: int,
+               entity_id: Union[int, list[int]],
                options: dict[str, Any] = None) -> bool:
         """
-        Delete a row from a model using its ID
+        Delete one or multiple rows from a model using the object IDs
 
-        :param entity_id: The object ID to delete
+        :param entity_id: The object IDs to delete
         :param options: Dictionary with options to use
-        :return: True if the record was deleted
+        :return: True if the records were deleted
         """
+        # Set options
         if options is None:
             options = {}
         # Request data and get results
@@ -378,8 +437,24 @@ class Model(object):
                                      options=options)
         return results
 
-    def set_options_language(self,
-                             options: dict) -> Optional[str]:
+    def _set_active(self,
+                    filters: list[Union[BooleanOperator, Filter]],
+                    is_active: ActiveStatusChoice = ActiveStatusChoice.NOT_SET,
+                    ) -> None:
+        """
+        Add a new filter for active records
+
+        :param filters: List of filters used for searching the data
+        :return: None
+        """
+        if filters is not None:
+            if is_active == ActiveStatusChoice.BOTH:
+                filters.append(['active', CompareType.IN, is_active])
+            elif is_active != ActiveStatusChoice.NOT_SET:
+                filters.append(['active', CompareType.EQUAL, is_active])
+
+    def _set_options_language(self,
+                              options: dict) -> Optional[str]:
         """
         Apply the default language context to the options
 
@@ -394,9 +469,9 @@ class Model(object):
                 options['context'] = {'lang': self.api.language}
         return self.language
 
-    def set_order_by(self,
-                     options: dict,
-                     order: Optional[str]) -> dict:
+    def _set_order_by(self,
+                      options: dict,
+                      order: Optional[str]) -> dict:
         """
         Apply order for ordering results to the options
 
@@ -409,10 +484,10 @@ class Model(object):
             options['order'] = order
         return options
 
-    def set_pagination(self,
-                       options: dict,
-                       limit: Optional[int],
-                       offset: Optional[int]) -> dict:
+    def _set_pagination(self,
+                        options: dict,
+                        limit: Optional[int],
+                        offset: Optional[int]) -> dict:
         """
         Apply limit and offset for pagination to the options
 
@@ -440,6 +515,7 @@ class Model(object):
         :param options: Dictionary with options to use
         :return: Dictionary with the requested fields
         """
+        # Set options
         if options is None:
             options = {}
         # Limit results only to selected fields
@@ -449,7 +525,7 @@ class Model(object):
         if attributes is not None:
             options['attributes'] = attributes
         # Set language for translated fields
-        self.set_options_language(options=options)
+        self._set_options_language(options=options)
         # Request data and get results
         results = self.api.do_fields_get(fields=fields,
                                          options=options)
@@ -585,18 +661,31 @@ class Model(object):
     def execute(self,
                 method_name: str,
                 args: list[Any],
-                kwargs: dict[str, Any]) -> Any:
+                kwargs: dict[str, Any],
+                ignore_none_errors: bool = False) -> Any:
         """
         Execute a method on a model
 
         :param method_name: The method name to call
         :param args: Arguments list passed by position
         :param kwargs: Arguments dict passed by keyword
+        :param ignore_none_errors: Ignore error about None values
         :return: Method calling result data
         """
-        return self.api.do_execute(method_name=method_name,
-                                   args=args,
-                                   kwargs=kwargs)
+        try:
+            results = self.api.do_execute(method_name=method_name,
+                                          args=args,
+                                          kwargs=kwargs)
+        except xmlrpc.client.Fault as error:
+            none_message = 'cannot marshal None unless allow_none is enabled'
+            if (ignore_none_errors and
+                    error.faultCode == 1 and
+                    none_message in error.faultString):
+                # Ignore errors about None
+                results = None
+            else:
+                raise
+        return results
 
     def get_message_subtype_id(self, subtype: str):
         """
